@@ -15,7 +15,7 @@ use Chrome::DevToolsProtocol::Transport;
 use Scalar::Util 'weaken', 'isweak';
 use Try::Tiny;
 
-our $VERSION = '0.32';
+our $VERSION = '0.33';
 our @CARP_NOT;
 
 =head1 NAME
@@ -223,6 +223,7 @@ sub add_listener( $self, $event, $callback ) {
     );
     $self->listener->{ $event } ||= [];
     push @{ $self->listener->{ $event }}, $listener;
+    weaken $self->listener->{ $event }->[-1];
     $listener
 }
 
@@ -236,11 +237,15 @@ Explicitly remove a listener.
 
 sub remove_listener( $self, $listener ) {
     # $listener->{event} can be undef during global destruction
-    if( my $event = $listener->{event} ) {
-        $self->listener->{ $event } ||= [];
-        @{$self->listener->{ $event }} = grep { $_ != $listener }
-                                         grep { defined $_ }
-                                         @{$self->listener->{ $event }};
+    if( my $event = $listener->event ) {
+        my $l = $self->listener->{ $event } ||= [];
+        @{$l} = grep { $_ != $listener }
+                grep { defined $_ }
+                @{$self->listener->{ $event }};
+        # re-weaken our references
+        for (0..$#$l) {
+            weaken $l->[$_];
+        };
     };
 }
 
@@ -474,16 +479,22 @@ sub on_response( $self, $connection, $message ) {
         };
 
         if( my $listeners = $self->listener->{ $response->{method} } ) {
+            @$listeners = grep { defined $_ } @$listeners;
             if( $self->_log->is_trace ) {
                 $self->log( 'trace', "Notifying listeners", $response );
             } else {
                 $self->log( 'debug', sprintf "Notifying listeners for '%s'", $response->{method} );
             };
-            for my $listener (@$listeners) { eval {
-                $listener->notify( $response );
+            for my $listener (@$listeners) {
+                eval {
+                    $listener->notify( $response );
                 };
                 warn $@ if $@;
             };
+            # re-weaken our references
+            for (0..$#$listeners) {
+                weaken $listeners->[$_];
+            };
 
             $handled++;
         };
@@ -779,7 +790,12 @@ has 'callback' => (
     is => 'ro',
 );
 
+has 'event' => (
+    is => 'ro',
+);
+
 around BUILDARGS => sub( $orig, $class, %args ) {
+    croak "Need an event" unless $args{ event };
     croak "Need a callback" unless $args{ callback };
     croak "Need a DevToolsProtocol in protocol" unless $args{ protocol };
     return $class->$orig( %args )
@@ -792,6 +808,7 @@ sub notify( $self, @info ) {
 sub unregister( $self ) {
     $self->protocol->remove_listener( $self )
         if $self->protocol; # it's a weak ref so it might have gone away already
+    undef $self->{protocol};
 }
 
 sub DESTROY {
